package com.ozs.web.controller.accountmanagment;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.IdempotenceUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.common.utils.stateSecrets.SM4Utils;
import com.ozs.entity.BaseCameraManagement;
import com.ozs.entity.BaseRailwayManagement;
import com.ozs.entity.BaseUser;
import com.ozs.entity.MonitorSystem;
import com.ozs.entity.MsgAlarm;
import com.ozs.entity.MsgAlarmExt;
import com.ozs.entity.MsgAlarmFrequency;
import com.ozs.entity.vo.Files;
import com.ozs.entity.vo.MsgAlarmTemporary;
import com.ozs.entity.vo.MsgAlarmVo;
import com.ozs.entity.vo.ReqMsgAlarmVo;
import com.ozs.service.BaseCameraManagementService;
import com.ozs.service.BaseRailwayManagementService;
import com.ozs.service.BaseUserService;
import com.ozs.service.MsgAlarmExtService;
import com.ozs.service.MsgAlarmFrequencyService;
import com.ozs.service.MsgAlarmService;
import com.ozs.service.MsgAlarmTemporaryService;
import com.ozs.service.SvcAddressService;
import com.ozs.system.service.ISysDictTypeService;
import com.ozs.web.controller.monitor.SysUserOnlineController;
import com.ozs.web.controller.websocket.WebSocketCommon;
import com.ozs.web.core.util.CameraUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @author Administrator
 */
@RestController
@RequestMapping("/service/msgAlarmTemporary")
public class MsgAlarmTemporaryController extends BaseController {

    public static final String PATTERN = "^([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)$";
    public static final String PATTERNS = "^[0-9]*$";

    @Autowired
    private IdempotenceUtils idempotenceUtils;

    @Autowired
    private MsgAlarmTemporaryService msgAlarmTemporaryService;

    @Autowired
    private BaseRailwayManagementService baseRailwayManagementService;

    @Autowired
    SvcAddressService svcAddressService;
    @Autowired
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    SysUserOnlineController sysUserOnlineController;
    @Autowired
    private BaseUserService baseUserService;
    @Autowired
    private MsgAlarmExtService msgAlarmExtService;
    @Autowired
    WebSocketCommon webSocketCommon;
    @Autowired
    private MsgAlarmService msgAlarmService;
    @Autowired
    private MsgAlarmFrequencyService msgAlarmFrequencyService;
    @Autowired
    private ISysDictTypeService dictTypeService;



    /**
     * 报警信息临时表分页
     *
     * @param msgAlarmVo
     * @return
     */
    @ApiOperation(value = "报警信息临时表分页")
    @PostMapping("/list")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult selectMsgAlarm(@RequestBody MsgAlarmVo msgAlarmVo) {
        IPage<MsgAlarmTemporary> page = msgAlarmTemporaryService.page(new Page<>(msgAlarmVo.getPageNum(), msgAlarmVo.getPageSize()), null);
        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<MsgAlarmTemporary> dto1 = page.getRecords().stream().map(o -> {
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(o.getAlarmRailway())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, o.getAlarmRailway());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                if (!StringUtils.isEmptySunhh(baseRailwayManagement) && !StringUtils.isEmptySunhh(baseRailwayManagement.getRailwayName())) {
                    o.setRailwayName(baseRailwayManagement.getRailwayName());
                }
                String mils = AppendUtils.stringAppend(o.getAlarmMile());
                o.setAlarmMiles(mils);
                o.setLineDirName(o.getLineDir() == 1 ? "上行" : "下行");
                o.setStareName(o.getLineDir() == 1 ? "未使用" : "已使用");
                o.setAlarmTypeStr(o.getAlarmType() == 1 ? "泥石流" : (o.getAlarmType() == 2 ? "异物侵线" : (o.getAlarmType() == 3 ? "断轨监测" : "未知")));
                List<SysDictData> data = dictTypeService.selectDictDataByType("sys_alarm_type");
                data.stream().forEach(sysDictData -> {
                    if (sysDictData.getDictValue().equals(o.getAlarmType().toString())) {
                        o.setAlarmTypeName(sysDictData.getDictLabel());
                    }
                });
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }


    @PostMapping("/saveMsgAlarmTemporary")
    @ApiOperation("新增报警信息临时表信息")
    @Log(title = "报警信息临时表管理", businessType = BusinessType.INSERT)
    public AjaxResult saveMsgAlarmTemporary(@RequestBody MsgAlarmTemporary msgAlarmTemporary) {
        String idempotence = idempotenceUtils.getIdempotence(msgAlarmTemporary.getAlarmId());
        if (StringUtils.isNotEmpty(idempotence)) {
            return error("你的网络卡顿,请不要一直添加!!!");
        }
        idempotenceUtils.createIdempotence(msgAlarmTemporary.getAlarmId());
        LambdaQueryWrapper<MsgAlarmTemporary> lw = new LambdaQueryWrapper<>();
        if (!ObjectUtils.isEmpty(msgAlarmTemporary.getAlarmId())) {
            lw.eq(MsgAlarmTemporary::getAlarmId, msgAlarmTemporary.getAlarmId());
            MsgAlarmTemporary msgAlarmTemporary1 = msgAlarmTemporaryService.getOne(lw);
            if (!ObjectUtils.isEmpty(msgAlarmTemporary1)) {
                idempotenceUtils.deleteIdempotence(msgAlarmTemporary.getAlarmId());
                return error("报警唯一标识已存在,不能重复添加");
            }
        }
        if (msgAlarmTemporaryService.save(msgAlarmTemporary)) {
            idempotenceUtils.deleteIdempotence(msgAlarmTemporary.getAlarmId());
            return AjaxResult.success();
        }
        idempotenceUtils.deleteIdempotence(msgAlarmTemporary.getAlarmId());
        return AjaxResult.error();
    }


    @GetMapping("/alarm/{id}")
    @ApiOperation("报警推送假数据")
    public String alarm(@PathVariable Long id) {
        JSONObject jsonObject = new JSONObject();
        MsgAlarmTemporary msgAlarmTemporary = msgAlarmTemporaryService.getById(id);
        if (msgAlarmTemporary.getStare()==2){
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "数据已模拟,不能再次使用");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        ReqMsgAlarmVo reqMsgAlarmVo = new ReqMsgAlarmVo();
        com.ozs.common.utils.bean.BeanUtils.copyProperties(msgAlarmTemporary, reqMsgAlarmVo);
        reqMsgAlarmVo.setAlarmTime(msgAlarmTemporary.getAlarmTime().getTime());
        Files files=new Files();
        files.setFileType(1);
        files.setFileUrl(msgAlarmTemporary.getFileUrl());
        Files[] files2=new Files[1];
        files2[0]=files;
        reqMsgAlarmVo.setFiles(files2);
        if (!ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmAttr())) {
            if (!reqMsgAlarmVo.getAlarmAttr().matches(PATTERN)) {
                jsonObject.put("resultCode", 0);
                jsonObject.put("message", "失败");
                jsonObject.put("data", "你填写的报警病害属性不符合规则!!!");
                return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
            }
        }
        if (!reqMsgAlarmVo.getAlarmMile().toString().matches(PATTERNS)) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警里程位置填写错误");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (reqMsgAlarmVo.getLineDir() != 1 && reqMsgAlarmVo.getLineDir() != 2) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "行别填写错误");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmRailway())) {
            queryWrapper.eq(BaseRailwayManagement::getRailwayCode, reqMsgAlarmVo.getAlarmRailway());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(baseRailwayManagement)) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "填写的报警线路编码不存在");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmId())) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警的唯一标识不能为空");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmTime())) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警时间不能为空");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (reqMsgAlarmVo.getAlarmType() < 1 || reqMsgAlarmVo.getAlarmType() > 3) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警病害类型填写错误");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (!reqMsgAlarmVo.getAlarmLevel().toString().matches(PATTERNS)) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警级别填写错误");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        if (reqMsgAlarmVo.getAlarmTime().toString().length() == 10) {
            reqMsgAlarmVo.setAlarmTime(reqMsgAlarmVo.getAlarmTime() * 1000);
        }

        LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmMile())) {
            wrapper.le(BaseCameraManagement::getBeginMile, reqMsgAlarmVo.getAlarmMile());
            wrapper.ge(BaseCameraManagement::getEndMile, reqMsgAlarmVo.getAlarmMile());
            wrapper.eq(BaseCameraManagement::getRailwayCode, reqMsgAlarmVo.getAlarmRailway());
            wrapper.eq(BaseCameraManagement::getLineDir, reqMsgAlarmVo.getLineDir());
        }
        List<BaseCameraManagement> baseCameraManagementList = baseCameraManagementService.list(wrapper);
        if (baseCameraManagementList.size() <= 0) {
            jsonObject.put("resultCode", 0);
            jsonObject.put("message", "失败");
            jsonObject.put("data", "报警里程位置没有可用的相机!!!");
            return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
        }
        LambdaQueryWrapper<MsgAlarm> lw = new LambdaQueryWrapper<MsgAlarm>();
        if (!ObjectUtils.isEmpty(reqMsgAlarmVo.getAlarmId())) {
            lw.eq(MsgAlarm::getAlarmId, reqMsgAlarmVo.getAlarmId());
        }
        MsgAlarm alarm = msgAlarmService.getOne(lw);
        BaseUser admin = baseUserService.getUserName("admin");
        MsgAlarmFrequency msgAlarmVice = new MsgAlarmFrequency();
        if (ObjectUtils.isEmpty(alarm)) {
            MsgAlarm msgAlarm = new MsgAlarm();
            BeanUtils.copyProperties(reqMsgAlarmVo, msgAlarm);
            BeanUtils.copyProperties(reqMsgAlarmVo, msgAlarmVice);
            msgAlarm.setCreateBy(admin.getUserId());
            msgAlarm.setUpdateBy(admin.getUserId());
            msgAlarm.setAlarmTime(new Date(reqMsgAlarmVo.getAlarmTime()));
            msgAlarm.setRailwayCode(reqMsgAlarmVo.getAlarmRailway());
            msgAlarm.setSource("source");
            msgAlarmService.save(msgAlarm);
            Files[] files1 = reqMsgAlarmVo.getFiles();
            if (files1.length > 0) {
                for (int i = 0; i < files1.length; i++) {
                    if (ObjectUtils.isEmpty(files1[i].fileUrl)) {
                        jsonObject.put("resultCode", 0);
                        jsonObject.put("message", "失败");
                        jsonObject.put("data", "上传文件地址不能为空");
                        return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
                    }
                    if (files1[i].fileType < 1 || files1[i].fileType > 3) {
                        jsonObject.put("resultCode", 0);
                        jsonObject.put("message", "失败");
                        jsonObject.put("data", "文件类型填写错误");
                        return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
                    }
                    MsgAlarmExt msgAlarmExt = new MsgAlarmExt();
                    msgAlarmExt.setAlarmId(msgAlarm.getAlarmId());
                    msgAlarmExt.setAlarmAttPath(files1[i].fileUrl);
                    msgAlarmExt.setAlarmAttType(files1[i].fileType);
                    msgAlarmExtService.save(msgAlarmExt);
                }
            }
            msgAlarmVice.setCreateBy(admin.getUserId());
            msgAlarmVice.setUpdateBy(admin.getUserId());
            msgAlarmVice.setRailwayCode(reqMsgAlarmVo.getAlarmRailway());
            msgAlarmVice.setAlarmTime(new Date(reqMsgAlarmVo.getAlarmTime()));
            msgAlarmFrequencyService.save(msgAlarmVice);
            webSocketCommon.publish(msgAlarm);
            for (BaseCameraManagement baseCameraManagement : baseCameraManagementList) {
                String invite = CameraUtil.invite(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel());
                if ("304".equals(invite) || "200".equals(invite)) {
                    String start = HttpUtils.sendGet(CameraUtil.startRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
                    if (start.startsWith(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel())) {
                        baseCameraManagement.setTaskId(start);
                        baseCameraManagement.setUpdateBy(admin.getUserId());
                    } else {
                        jsonObject.put("resultCode", 0);
                        jsonObject.put("message", "失败");
                        jsonObject.put("data", "视频录制失败:请确认流是否存在!!!");
                        return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
                    }
                    if (!baseCameraManagementService.updateById(baseCameraManagement)) {
                        jsonObject.put("resultCode", 0);
                        jsonObject.put("message", "失败");
                        jsonObject.put("data", "开启录制视频失败!!!");
                        return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
                    }
                } else {
                    jsonObject.put("resultCode", 0);
                    jsonObject.put("message", "失败");
                    jsonObject.put("data", "视频邀约失败,视频中心没有该相机编码或通道编码的摄像头!!!");
                    return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
                }
            }

        } else {
            BeanUtils.copyProperties(reqMsgAlarmVo, msgAlarmVice);
            msgAlarmVice.setCreateBy(admin.getUserId());
            msgAlarmVice.setUpdateBy(admin.getUserId());
            msgAlarmVice.setAlarmTime(new Date(reqMsgAlarmVo.getAlarmTime()));
            msgAlarmVice.setRailwayCode(reqMsgAlarmVo.getAlarmRailway());
            msgAlarmFrequencyService.save(msgAlarmVice);
        }
        jsonObject.put("resultCode", 1);
        jsonObject.put("message", "ok");
        jsonObject.put("data", null);
        msgAlarmTemporary.setStare(2);
        msgAlarmTemporaryService.updateById(msgAlarmTemporary);
        return SM4Utils.encryptData_ECB(JSONObject.toJSONString(jsonObject), "4370780c9a8c43e5");
    }
}
